@@ -1,5 +1,6 @@
 /*
  * Copyright (c) 2003 Riccardo Persichetti <riccardo.persichetti@tin.it>
+ * Copyright (c) 2010 Florian Rivoal <frivoal@xfce.org>
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -32,7 +33,7 @@
 #include "memswap.h"
 
 
-#if defined(__linux__)
+#if defined(__linux__) || defined(__FreeBSD_kernel__)
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
@@ -47,6 +48,7 @@ static char MemInfoBuf[MEMINFOBUFSIZE];
 
 static unsigned long MTotal = 0;
 static unsigned long MFree = 0;
+static unsigned long MBuffers = 0;
 static unsigned long MCached = 0;
 static unsigned long MUsed = 0;
 static unsigned long STotal = 0;
@@ -57,8 +59,8 @@ gint read_memswap(gulong *mem, gulong *swap, gulong *MT, gulong *MU, gulong *ST,
 {
     int fd;
     size_t n;
-    int o_MTotal, o_MFree, o_MCached, o_STotal, o_SFree;
-    char *b_MTotal, *b_MFree, *b_MCached, *b_STotal, *b_SFree;
+    int o_MTotal, o_MFree, o_MBuffers, o_MCached, o_STotal, o_SFree;
+    char *b_MTotal, *b_MFree, *b_MBuffers, *b_MCached, *b_STotal, *b_SFree;
 
     if ((fd = open("/proc/meminfo", O_RDONLY)) < 0)
     {
@@ -83,6 +85,10 @@ gint read_memswap(gulong *mem, gulong *swap, gulong *MT, gulong *MU, gulong *ST,
     if (b_MFree)
         o_MFree = sscanf(b_MFree + strlen("MemFree"), ": %lu", &MFree);
 
+    b_MBuffers = strstr(MemInfoBuf, "Buffers");
+    if (b_MBuffers)
+        o_MBuffers = sscanf(b_MBuffers + strlen("Buffers"), ": %lu", &MBuffers);
+
     b_MCached = strstr(MemInfoBuf, "Cached");
     if (b_MCached)
         o_MCached = sscanf(b_MCached + strlen("Cached"), ": %lu", &MCached);
@@ -95,7 +101,7 @@ gint read_memswap(gulong *mem, gulong *swap, gulong *MT, gulong *MU, gulong *ST,
     if (b_SFree)
         o_SFree = sscanf(b_SFree + strlen("SwapFree"), ": %lu", &SFree);
 
-    MFree += MCached;
+    MFree += MCached + MBuffers;
     MUsed = MTotal - MFree;
     SUsed = STotal - SFree;
     *mem = MUsed * 100 / MTotal;
